"""
assign2.py:
Using SCOP2 for solving an assignment problem under linear constraints.
Copyright (c) by Joao Pedro PEDROSO and Mikio KUBO, 2011

Example 2 (Generalized Assignment Problem):
Three jobs (0,1,2) must be assigned to five workers (A,B,C,D,E).
The numbers of workers that must be assigned to jobs 0,1 and 2 are 1,1 and 2, respectively. 
The cost matrix is represented by the list of lists  
Cost=[[15, 20, 30],
[7, 15, 12],
[25,10,13],
[15,18,3],
[5,12,17]]
where rows are workers, and columns are jobs.
Find the minimum cost assignment of workers to jobs.

"""
from scop import *

m=Model()
workers=['A','B','C','D','E']
Jobs   =[0,1,2]
Cost={ ('A',0):15, ('A',1):20, ('A',2):30,
       ('B',0): 7, ('B',1):15, ('B',2):12,
       ('C',0):25, ('C',1):10, ('C',2):13,
       ('D',0):15, ('D',1):18, ('D',2): 3,
       ('E',0): 5, ('E',1):12, ('E',2):17
       }
LB={0: 1,
    1: 2,
    2: 2
    }
x={}
for i in workers:
    x[i]=m.addVariable(name=i,domain=Jobs)
LBC={} #dictionary for keeping lower bound constraints
for j in Jobs:
    LBC[j]=Linear('LB{0}'.format(j),'inf',LB[j],'>=')
    for i in workers:
        LBC[j].addTerms(1,x[i],j)
    m.addConstraint(LBC[j])
obj=Linear('obj')
for i in workers:
    for j in [0,1,2]:
        obj.addTerms(Cost[i,j],x[i],j)
m.addConstraint(obj)
m.Params.TimeLimit=1
sol,violated=m.optimize()
print(m)

print('solution')
for x in sol:
    print (x,sol[x])
print ('violated constraint(s)')
for v in violated:
    print (v,violated[v])

#for i in workers:
#    print i,x[i].value
    





    

