"""
gcp-scop.py:
Using SCOP for solving the graph coloring problem.
Copyright (c) by Joao Pedro PEDROSO and Mikio KUBO, 2011
"""
from scop import *

m=Model()

K=3
nodes=["n{0}".format(i) for i in range(6)]
adj=[[2],[3],[0,3,4,5],[1,2,5],[2],[2,3]]
n=len(nodes)

varlist=m.addVariables(nodes,range(K))

for i in range(n):
    for j in adj[i]:
        if i<j:
            con1=Alldiff("alldiff_{0}_{1}".format(i,j),[varlist[i],varlist[j]],"inf")      
            m.addConstraint(con1)
            
m.Params.TimeLimit=1
sol,violated=m.optimize()

print (m)

print ("solution")
for x in sol:
    print (x,sol[x])
print ("violated constraint(s)")
for v in violated:
    print (v,violated[v])

##number of variables = 6 
##number of constraints= 6
##variable n0:[0, 1, 2]
##variable n1:[0, 1, 2]
##variable n2:[0, 1, 2]
##variable n3:[0, 1, 2]
##variable n4:[0, 1, 2]
##variable n5:[0, 1, 2]
##alldiff02: weight= inf  type=alldiff n0 n2 ; 
##alldiff13: weight= inf  type=alldiff n1 n3 ; 
##alldiff23: weight= inf  type=alldiff n2 n3 ; 
##alldiff24: weight= inf  type=alldiff n2 n4 ; 
##alldiff25: weight= inf  type=alldiff n2 n5 ; 
##alldiff35: weight= inf  type=alldiff n3 n5 ; 
##
##solution
##n0  0

##n1  0

##n2  1

##n3  2

##n4  2

##n5  0
##violated constraint(s)


               




