"""
gpp-scop.py:
Using SCOP for solving the graph partitioning problem
Copyright (c) by Joao Pedro PEDROSO and Mikio KUBO, 2011
"""
from scop import *

m=Model()

nodes=["n%i"%i for i in range(6)]
adj=[[1,4],[0,2,4],[1],[4,5],[0,1,3,5],[3,4]]
n=len(nodes)

varlist=m.addVariables(nodes,[0,1])

con1=Linear("constraint","inf",n//2,"=")
for i in range(len(nodes)):
    con1.addTerms(1,varlist[i],1)
m.addConstraint(con1)

##con2={}
##for i in range(n):
##    for j in adj[i]:
##        con2[i,j]= Quadratic( "obj_%s_%s"%(i,j) )
##        con2[i,j].addTerms(1,varlist[i],1,varlist[j],0)
##        con2[i,j].addTerms(1,varlist[i],0,varlist[j],1)
##        m.addConstraint(con2[i,j])

con2=Quadratic( "obj")
for i in range(n):
    for j in adj[i]:
        con2.addTerms(1,varlist[i],1,varlist[j],0)
        con2.addTerms(1,varlist[i],0,varlist[j],1)
m.addConstraint(con2)

print (m)

m.Params.TimeLimit=1
sol,violated=m.optimize()

print ("solution")
for x in sol:
    print (x,sol[x])
print ("violated constraint(s)")
for v in violated:
    print (v,violated[v])

##number of variables = 6
##number of constraints= 2
##variable n0:[0, 1]
##variable n1:[0, 1]
##variable n2:[0, 1]
##variable n3:[0, 1]
##variable n4:[0, 1]
##variable n5:[0, 1]
##constraint: weight= inf type=linear 1(n0,1) 1(n1,1) 1(n2,1) 1(n3,1) 1(n4,1) 1(n5,1) =3
##obj: weight= 1 type=quadratic 1(n0,1) (n1,0) 1(n0,0) (n1,1) 1(n0,1) (n4,0) 1(n0,0) (n4,1) 1(n1,1) (n0,0) 1(n1,0) (n0,1) 1(n1,1) (n2,0) 1(n1,0) (n2,1) 1(n1,1) (n4,0) 1(n1,0) (n4,1) 1(n2,1) (n1,0) 1(n2,0) (n1,1) 1(n3,1) (n4,0) 1(n3,0) (n4,1) 1(n3,1) (n5,0) 1(n3,0) (n5,1) 1(n4,1) (n0,0) 1(n4,0) (n0,1) 1(n4,1) (n1,0) 1(n4,0) (n1,1) 1(n4,1) (n3,0) 1(n4,0) (n3,1) 1(n4,1) (n5,0) 1(n4,0) (n5,1) 1(n5,1) (n3,0) 1(n5,0) (n3,1) 1(n5,1) (n4,0) 1(n5,0) (n4,1) <=0
##
##solution
##n0  0

##n1  0

##n2  0

##n3  1

##n4  1

##n5  1
##violated constraint(s)
##obj  4

