"""
mkp-scop.py:
Using SCOP for solving the multi-constrained knapsack problem
Copyright (c) by Joao Pedro PEDROSO and Mikio KUBO, 2011
"""
from scop import *

model=Model()

v=[16,19,23,28]
a=[[2,3,4,5],[3000,3500,5100,7200]]
b=[7,10000]
n=len(v)
m=len(b)
items=["item{0}".format(j) for j in range(n)]
varlist=model.addVariables(items,[0,1])
for i in range(m):
    con1=Linear("mkp_{0}".format(i),"inf",b[i])
    for j in range(n):
        con1.addTerms(a[i][j],varlist[j],1)
    model.addConstraint(con1)

con2=Linear("obj",1,sum(v),">=")
for j in range(n):
    con2.addTerms(v[j],varlist[j],1)
model.addConstraint(con2)

model.Params.TimeLimit=1
sol,violated=model.optimize()

print (model)

print ("solution")
for x in sol:
    print (x,sol[x])
print ("violated constraint(s)")
for v in violated:
    print (v,violated[v])

    

##['item0', 'item1', 'item2', 'item3']
##number of variables = 4 
##number of constraints= 3
##variable item0:[0, 1]
##variable item1:[0, 1]
##variable item2:[0, 1]
##variable item3:[0, 1]
##mkp0: weight= inf type=linear 2(item0,1) 3(item1,1) 4(item2,1) 5(item3,1) <=7
##mkp1: weight= inf type=linear 3000(item0,1) 3500(item1,1) 5100(item2,1) 7200(item3,1) <=10000
##obj: weight= 1 type=linear 16(item0,1) 19(item1,1) 23(item2,1) 28(item3,1) >=86
##
##solution
##item2  1

##item3  0
##item0  0

##item1  1

##violated constraint(s)
##obj  44

        
    
