"""
qap-scop.py:
Using SCOP for solving the quadratic assignment problem
Copyright (c) by Joao Pedro PEDROSO and Mikio KUBO, 2011
"""
from scop import *
m=Model()

n=3
d=[[0,2,3],[2,0,1],[3,1,0]]
f=[[0,5,1],[5,0,2],[1,2,0]]

nodes=["n{0}".format(i) for i in range(n)]

varlist=m.addVariables(nodes,range(n))

con1=Alldiff("AD",varlist,"inf")
m.addConstraint(con1)

obj=Quadratic("obj")
for i in range(n-1):
    for j in range(i+1,n):
        for k in range(n):
            for ell in range(n):
                if k !=ell:
                    obj.addTerms(f[i][j]*d[k][ell],varlist[i],k,varlist[j],ell)
m.addConstraint(obj)


m.Params.TimeLimit=1
sol,violated=m.optimize()

#print m


print ("solution")
for x in sol:
    print (x,sol[x])
print ("violated constraint(s)")
for v in violated:
    print (v,violated[v])

##number of variables = 3 
##number of constraints= 2
##variable n0:[0, 1, 2]
##variable n1:[0, 1, 2]
##variable n2:[0, 1, 2]
##AD: weight= inf  type=alldiff n0 n1 n2 ; 
##obj: weight= 1 type=quadratic 10(n0,0) (n1,1) 15(n0,0) (n1,2) 10(n0,1) (n1,0) 5(n0,1) (n1,2) 15(n0,2) (n1,0) 5(n0,2) (n1,1) 2(n0,0) (n2,1) 3(n0,0) (n2,2) 2(n0,1) (n2,0) 1(n0,1) (n2,2) 3(n0,2) (n2,0) 1(n0,2) (n2,1) 4(n1,0) (n2,1) 6(n1,0) (n2,2) 4(n1,1) (n2,0) 2(n1,1) (n2,2) 6(n1,2) (n2,0) 2(n1,2) (n2,1) <=0
##
##solution
##n0  2

##n1  1

##n2  0
##violated constraint(s)
##obj  12
    
