"""
ssp-scop.py:
Using SCOP for solving the stable set problem
Copyright (c) by Joao Pedro PEDROSO and Mikio KUBO, 2011
"""
from scop import *

m=Model()

nodes=["n{0}".format(i) for i in range(6)]
adj=[[2],[3],[0,3,4,5],[1,2,5],[2],[2,3]]
n=len(nodes)

varlist=m.addVariables(nodes,[0,1])

for i in range(n):
    for j in adj[i]:
        if i<j:
            con1=Linear("constraint{0}_{1}".format(i,j),"inf",1)
            con1.addTerms(1,varlist[i],1)
            con1.addTerms(1,varlist[j],1)
            m.addConstraint(con1)

obj=Linear("obj",1,n,">=")
for i in range(n):
    obj.addTerms(1,varlist[i],1)
m.addConstraint(obj)

m.Params.TimeLimit=1
sol,violated=m.optimize()

print (m)

print ("solution")
for x in sol:
    print (x,sol[x])
print ("violated constraint(s)")
for v in violated:
    print (v,violated[v])


##number of variables = 6 
##number of constraints= 7
##variable n0:[0, 1]
##variable n1:[0, 1]
##variable n2:[0, 1]
##variable n3:[0, 1]
##variable n4:[0, 1]
##variable n5:[0, 1]
##constraint02: weight= inf type=linear 1(n0,1) 1(n2,1) <=1
##constraint13: weight= inf type=linear 1(n1,1) 1(n3,1) <=1
##constraint23: weight= inf type=linear 1(n2,1) 1(n3,1) <=1
##constraint24: weight= inf type=linear 1(n2,1) 1(n4,1) <=1
##constraint25: weight= inf type=linear 1(n2,1) 1(n5,1) <=1
##constraint35: weight= inf type=linear 1(n3,1) 1(n5,1) <=1
##obj: weight= 1 type=linear 1(n0,1) 1(n1,1) 1(n2,1) 1(n3,1) 1(n4,1) 1(n5,1) >=6
##
##solution
##n0  1

##n1  1

##n2  0

##n3  0

##n4  1

##n5  1
##violated constraint(s)
##obj  2





