"""
tsp-scop.py:
Using SCOP for solving the traveling salesman problem
Copyright (c) by Joao Pedro PEDROSO and Mikio KUBO, 2011
"""
from scop import *

m=Model()

cities=["T","L","M","R","B"]
d=[[0,476,774,434,408],[476,0,784,894,569],[774,784,0,852,1154],[434,894,852,0,569],[408,569,1154,569,0]]
n=len(cities)

varlist=m.addVariables(cities,range(n))

con1=Alldiff("AD",varlist,"inf")
m.addConstraint(con1)

obj=Quadratic("obj")
for i in range(n):
    for j in range(n):
        if i!=j:
            for k in range(n):
                if k ==n-1:
                    ell=0
                else:
                    ell=k+1
                obj.addTerms(d[i][j],varlist[i],k,varlist[j],ell)
m.addConstraint(obj)

##obj={}
##for i in range(n):
##    for j in range(n):
##        if i!=j:
##            for k in range(n):
##                if k ==n-1:
##                    ell=0
##                else:
##                    ell=k+1
##                obj[i,j,k,ell]=Quadratic("obj_%s_%s_%s_%s"%(i,j,k,ell))
##                obj[i,j,k,ell].addTerms(d[i][j],varlist[i],k,varlist[j],ell)
##                m.addConstraint(obj[i,j,k,ell])

m.Params.TimeLimit=1
sol,violated=m.optimize()

print (m)

print ("solution")
for x in sol:
    print (x,sol[x])
print ("violated constraint(s)")
for v in violated:
    print (v,violated[v])
          

##number of variables = 5 
##number of constraints= 2
##variable T:[0, 1, 2, 3, 4]
##variable L:[0, 1, 2, 3, 4]
##variable M:[0, 1, 2, 3, 4]
##variable R:[0, 1, 2, 3, 4]
##variable B:[0, 1, 2, 3, 4]
##AD: weight= inf  type=alldiff T L R M B ; 
##obj: weight= 1 type=quadratic 476(T,0) (L,1) 476(T,1) (L,2) 476(T,2) (L,3) 476(T,3) (L,4) 476(T,4) (L,0) 774(T,0) (M,1) 774(T,1) (M,2) 774(T,2) (M,3) 774(T,3) (M,4) 774(T,4) (M,0) 434(T,0) (R,1) 434(T,1) (R,2) 434(T,2) (R,3) 434(T,3) (R,4) 434(T,4) (R,0) 408(T,0) (B,1) 408(T,1) (B,2) 408(T,2) (B,3) 408(T,3) (B,4) 408(T,4) (B,0) 476(L,0) (T,1) 476(L,1) (T,2) 476(L,2) (T,3) 476(L,3) (T,4) 476(L,4) (T,0) 784(L,0) (M,1) 784(L,1) (M,2) 784(L,2) (M,3) 784(L,3) (M,4) 784(L,4) (M,0) 894(L,0) (R,1) 894(L,1) (R,2) 894(L,2) (R,3) 894(L,3) (R,4) 894(L,4) (R,0) 569(L,0) (B,1) 569(L,1) (B,2) 569(L,2) (B,3) 569(L,3) (B,4) 569(L,4) (B,0) 774(M,0) (T,1) 774(M,1) (T,2) 774(M,2) (T,3) 774(M,3) (T,4) 774(M,4) (T,0) 784(M,0) (L,1) 784(M,1) (L,2) 784(M,2) (L,3) 784(M,3) (L,4) 784(M,4) (L,0) 852(M,0) (R,1) 852(M,1) (R,2) 852(M,2) (R,3) 852(M,3) (R,4) 852(M,4) (R,0) 1154(M,0) (B,1) 1154(M,1) (B,2) 1154(M,2) (B,3) 1154(M,3) (B,4) 1154(M,4) (B,0) 434(R,0) (T,1) 434(R,1) (T,2) 434(R,2) (T,3) 434(R,3) (T,4) 434(R,4) (T,0) 894(R,0) (L,1) 894(R,1) (L,2) 894(R,2) (L,3) 894(R,3) (L,4) 894(R,4) (L,0) 852(R,0) (M,1) 852(R,1) (M,2) 852(R,2) (M,3) 852(R,3) (M,4) 852(R,4) (M,0) 569(R,0) (B,1) 569(R,1) (B,2) 569(R,2) (B,3) 569(R,3) (B,4) 569(R,4) (B,0) 408(B,0) (T,1) 408(B,1) (T,2) 408(B,2) (T,3) 408(B,3) (T,4) 408(B,4) (T,0) 569(B,0) (L,1) 569(B,1) (L,2) 569(B,2) (L,3) 569(B,3) (L,4) 569(B,4) (L,0) 1154(B,0) (M,1) 1154(B,1) (M,2) 1154(B,2) (M,3) 1154(B,3) (M,4) 1154(B,4) (M,0) 569(B,0) (R,1) 569(B,1) (R,2) 569(B,2) (R,3) 569(B,3) (R,4) 569(B,4) (R,0) <=0
##
##solution
##B  2
##R  0

##M  4

##T  1

##L  3

##violated constraint(s)
##obj  3047

                
